<?php

namespace App\Http\Controllers\Auth;

use App\Enums\OriginEnum;
use Illuminate\Http\Request;
use App\Services\Auth\AuthService;
use App\Traits\ApiResponse;
use App\Http\Requests\LoginRequest; // Import the LoginRequest
use App\Http\Controllers\Controller;
use App\Http\Requests\RegistrationRequest;
use App\Services\Contracts\TokenServiceInterface;
use App\Services\UserCacheService;
use App\Services\DataLoaderService;
use App\Traits\AuthenticateUser;
use Illuminate\Support\Facades\Cookie;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Log;


class AuthController extends Controller
{
    use ApiResponse;
    use AuthenticateUser;

    protected $authService;
    protected $dataLoaderService;
    protected $tokenService;

    public function __construct(AuthService $authService, DataLoaderService  $dataLoaderService, TokenServiceInterface $tokenService)
    {
        $this->authService = $authService;
        $this->dataLoaderService = $dataLoaderService;
        $this->tokenService = $tokenService;
    }


    /**
     * Handle the user registration.
     *
     * @param  \App\Http\Requests\RegistrationRequest  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function register(RegistrationRequest $request)
    {
        try {
            // Retrieve the validated input data
            $validatedData = $request->validated();

            // $result = $this->authService->register($data);

            // Log the user in
            // $tokens = $this->performLogin($user);
            $tokens = $this->tokenService->generateStaticToken();

            // Return a success response using ApiResponse
            // return $this->success(
            //     ['user' => $validatedData],
            //     'Registration successful.',
            //     201
            // )->withCookie($tokens['accessTokenCookie'])->withCookie($tokens['refreshTokenCookie']);

            $accessTokenCookie = Cookie::make('accessToken', $tokens, 60 * 1, '/', null, false, true);
            $refreshTokenCookie = Cookie::make('refreshToken', $tokens, 60 * 24, '/', null, false, true);

            return $this->success(
                ['user' => $validatedData],
                'Registration successful.',
                201
            )->withCookie($accessTokenCookie)->withCookie($refreshTokenCookie);

        } catch (\Exception $e) {
            // Return an error response using ApiResponse
            return $this->error(
                'An error occurred during registration.',
                500,
                ['error' => $e->getMessage()]
            );
        }
    }


    /**
     * Handle user login.
     *
     * @param LoginRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(LoginRequest $request)
    {


        // Let the LoginRequest handle initial validation
        $credentials = $request->validated();

        // Authenticate the user
        $response = $this->authService->authenticate($credentials);

        // Retrieve the user from the cache
        $user = UserCacheService::getInstance()->getUser();


        // Destructure the user
        $userData = [
            'id' => $user['id'],
            'name' => $user['name'],
            'username' => $user['username'],
            'phone' => $user['phone'],
            'verification_status' => $user['verification_status'],
            'subscription_status' => $user['subscription_status']
        ];

        $isWeb = $credentials['origin'] === OriginEnum::WEB->value;
        $message = 'Login successful';
        // Handle successful authentication
        if ($response['success'] && $isWeb) {


            $tokens = $this->performLogin($user);

            return $this->success(
                $userData,
                $message
            )->withCookie($tokens['accessTokenCookie'])->withCookie($tokens['refreshTokenCookie']);
        } elseif ($response['success'] && !$isWeb) {
            $data = $this->dataLoaderService->loadDefaults($user, $credentials['origin']);

            return $this->success([
                'user' => $userData,
                'defaultRights' => $data['defaultRights'],
            ], $message);
        }

        // Handle login failure
        return $this->error($response['message'], 401);
    }


    public function checkSession(Request $request)
    {
        // Get the user from the request (attached by the middleware)
        if ($user = $request->user) {
            // Destructure the user
            $userData = [
                'id' => $user['id'],
                'name' => $user['name'],
                'username' => $user['username'],
                'verification_status' => $user['verification_status'],
            ];
            // Return the user data using the success method from ApiResponse trait
            return $this->success($userData, 'User is still logged in.');
        }

        // If no user (token is invalid or missing), return an error using the error method
        return $this->error('No active session found.', 401);
    }


    public function logout(Request $request)
    {
        // Simulate an unhandled exception
        // throw new \Exception('This is a test exception to verify the global handler.');

        // For web clients (with cookies)
        if ($request->hasCookie('accessToken') && $request->hasCookie('refreshToken')) {
            // Expire the cookies by setting the expiry to a past time
            $accessTokenCookie = cookie('accessToken', '', -1);  // Expire the access token cookie
            $refreshTokenCookie = cookie('refreshToken', '', -1); // Expire the refresh token cookie

            // Return success response with cookies cleared
            return $this->success([], 'Logged out successfully.')
                ->withCookie($accessTokenCookie)
                ->withCookie($refreshTokenCookie);
        }

        // For clients using Bearer JWT tokens (via headers or request body)
        try {
            // Invalidate the JWT token (if available in the Authorization header)
            $token = $request->bearerToken();

            if ($token) {
                // Invalidate the token to log the user out
                $this->tokenService->invalidateToken($token);
                // Return success message
                return $this->success([], 'Logged out successfully.');
            } else {
                // No Bearer token found in the request
                return $this->error('No token provided', 400);
            }
        } catch (\Exception $e) {
            // Handle any errors that occur during token invalidation
            return $this->error('Failed to log out', 500);
        }
    }

    public function refresh(Request $request)
    {
        $refreshToken = $request->token ?? $request->cookie('refreshToken');

        if (!$refreshToken) {
            return response()->json([
                'message' => 'No refresh token provided.',
            ], Response::HTTP_BAD_REQUEST);
        }
        try {
            $response = $this->tokenService->refreshToken($refreshToken);
            $successMsg = 'Token refreshed successfully.';

            if ($request->hasCookie('refreshToken')) {
                $accessTokenCookie = cookie('accessToken', $response, 60 * 1); // 1 hr expiry
                return $this->success([], $successMsg)
                    ->withCookie($accessTokenCookie);
            }
            return $this->success($response, $successMsg);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Token refresh failed.',
            ], Response::HTTP_UNAUTHORIZED);
        }
    }
}
