<?php

namespace App\Http\Controllers\CustomerEngagement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\Segment;
use App\Models\Account\User;
use Carbon\Carbon;
use DB;
use Validator;
use Auth;
use App\Http\Controllers\General\GeneralController;
use App\Http\Controllers\General\SmsController;

class CustomerRetargetingController extends Controller
{
    protected $generalController;
    protected $smsController;

    public function __construct(
        GeneralController $generalController,
        SmsController $smsController
    ){
        $this->generalController = $generalController;
        $this->smsController = $smsController;
    }

    private function validateRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => "required|unique:." . RETAILS_DB_VARIABLE .".segment,name",
            'description' => 'required',
            'visit_type' => 'sometimes|required_without:purchase_type',
            'purchase_type' => 'sometimes|required_without:visit_type',
            'visit_frequency' => 'required_with:visit_type',
            'purchase_frequency' => 'required_with:purchase_type',
            'visit_threshold' => 'required_if:visit_type,number of visits',
            'purchase_threshold' => 'required_if:purchase_type,purchase amount is',
        ], [
            'name.required' => 'The name field is required.',
            'name.unique' => 'The name has already been taken.',
            'description.required' => 'The description field is required.',
            'visit_type.required_without' => 'The visit field and purchase field cannot be both empty.',
            'purchase_type.required_without' => 'The visit field and purchase field cannot be both empty.',
            'visit_frequency.required_with' => 'The visit frequency field is required when visit field is provided.',
            'purchase_frequency.required_with' => 'The purchase frequency field is required when purchase field is provided.',
            'visit_threshold.required_if' => 'The visit threshold field is required when visit field is "number of visits".',
            'purchase_threshold.required_with' => 'The purchase threshold field is required when purchase type is "purchase amount is".',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first()], 400);
        }

        return null;
    }

    public function createSegment(Request $request)
    {
        $validationResult = $this->validateRequest($request);
            
        if ($validationResult) {
            return $validationResult;
        }

        $fields=[
            'name'=>$request->name,
            'description'=>$request->description,
            'visit_type'=>$request->visit_type,
            'visit_frequency'=>$request->visit_frequency,
            'visit_threshold'=>$request->visit_threshold,
            'purchase_type'=>$request->purchase_type,
            'purchase_frequency'=> $request->purchase_frequency,
            'purchase_threshold'=> $request->purchase_threshold,
            'default'=> STATUS,
            'tm'=>Carbon::now()->toTimeString(),
            'status'=> STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'date' => Carbon::now()->format('Y-m-d'),
            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'control_user_id' => STATUS,
            'control_id' => STATUS,
        ];

        $segment = Segment::create($fields);

        return response()->json(['message' => 'Segment created successfully', 'success' => true, 'data' => $segment], 200);
    }

    public function getSegment() {
        $data = Segment::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('status', STATUS)
        ->get();

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function frequency($frequency="show top 10") {
        
        if($frequency == "show top 10") {
            return 'LIMIT 10';
        }
        if($frequency == "show top 20") {
            return 'LIMIT 20';
        }
        if($frequency == "show top 50") {
            return 'LIMIT 50';
        }
        if($frequency == "show top 100") {
            return 'LIMIT 100';
        }

        return "";
    }


    public function threshold($frequency, $query, $threshold) {
        if($frequency == "equal to") {
            return "HAVING $query = $threshold";
        }
        if($frequency == "greater than") {
            return "HAVING $query > $threshold";
        }
        if($frequency == "less than") {
            return "HAVING $query < $threshold";
        }
        if($frequency == "less than or equal to") {
            return "HAVING $query > $threshold";
        }
        if($frequency == "greater than or equal to") {
            return "HAVING $query >= $threshold";
        }
        return "";
    }

    public function visit_type($type) {
        if($type=='low visit') {
            return "ASC";
        }

        if($type=='high visit') {
            return "DESC";
        }   

        return "ASC";
    }

    public function purchase_type($type) {
        if($type=='least purchase amount') {
            return "ASC";
        }

        if($type=='most purchase amount') {
            return "DESC";
        }

        return "ASC";
    }

    private function validateGetRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'from' => 'required',
            'to' => 'required',
            'visit_type' => 'sometimes|required_without:purchase_type',
            'purchase_type' => 'sometimes|required_without:visit_type',
            'visit_frequency' => 'required_with:visit_type',
            'purchase_frequency' => 'required_with:purchase_type',
            'visit_threshold' => 'required_if:visit_type,number of visits',
            'purchase_threshold' => 'required_if:purchase_type,purchase amount is',
        ], [
            'from.required' => 'The start date is required.',
            'to.required' => 'The end date is required.',
            'visit_type.required_without' => 'The visit field and purchase field cannot be both empty.',
            'purchase_type.required_without' => 'The visit field and purchase field cannot be both empty.',
            'visit_frequency.required_with' => 'The visit frequency field is required when visit field is provided.',
            'purchase_frequency.required_with' => 'The purchase frequency field is required when purchase field is provided.',
            'visit_threshold.required_if' => 'The visit threshold field is required when visit field is "number of visits".',
            'purchase_threshold.required_with' => 'The purchase threshold field is required when purchase type is "purchase amount is".',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first()], 400);
        }

        return null;
    }

    public function customerRetargeting($request) {

        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
     
        $visit_frequency = $this->frequency($request->visit_frequency);
        $visit_threshold = !empty($request->visit_threshold) ? $this->threshold($request->visit_frequency, "COUNT(DISTINCT t.token)", $request->visit_threshold) : "";

        $purchase_frequency = $this->frequency($request->purchase_frequency);
        $purchase_threshold = !empty($request->purchase_threshold) ? $this->threshold($request->purchase_frequency, "SUM(t.amount)", $request->purchase_threshold) : "";

        $visit_type = $this->visit_type($request->visit_type);
        $purchase_type = $this->purchase_type($request->purchase_type);

        $item_id = !empty($request->item_id) ? "AND t.item_id='$request->item_id'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND t.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        
        $query = "
            WITH visit AS 
            (
                SELECT 
                    c.name,
                    c.email,
                    c.phone,
                    t.coy,
                    COUNT(DISTINCT t.token) AS visits

                FROM " . TBL_PRODUCT_LOG . " t

                INNER JOIN " . TBL_CUSTOMERS . " c ON c.id = t.coy AND c.status = 0 AND c.client_id='$client_id' AND c.name != '" . WALK_IN_CUSTOMER . "' AND c.name != '" . DEFAULT_VENDOR . "'

                WHERE t.control_id = 0 AND t.client_id='$client_id' AND t.location_id='$location_id' $item_id $date
                GROUP BY c.name, t.coy
                $visit_threshold
                ORDER BY visits $visit_type $visit_frequency
            ),

            purchase AS 
            (
                SELECT
                    t.coy,
                    SUM(t.amount) AS purchase_amount,
                    MAX(t.amount) AS largest_purchase_amount

                FROM " . TBL_PRODUCT_LOG . " t

                INNER JOIN " . TBL_CUSTOMERS . " c ON c.id = t.coy AND c.status = 0 AND c.client_id='$client_id' AND c.name != '" . WALK_IN_CUSTOMER . "' AND c.name != '" . DEFAULT_VENDOR . "'

                WHERE t.control_id = 0 AND t.client_id='$client_id' AND t.location_id='$location_id' $item_id $date
                GROUP BY c.name, t.coy
                $purchase_threshold
                ORDER BY purchase_amount $purchase_type $purchase_frequency
            )

            SELECT 
                visit.name, 
                visit.email,
                visit.phone,
                visit.coy, 
                visit.visits, 
                purchase.purchase_amount,
                purchase.largest_purchase_amount
            FROM visit
            INNER JOIN purchase ON visit.coy=purchase.coy;
        ";

        $data = DB::select($query);

        return $data;
    }
    
    public function getCustomerRetargeting(Request $request) {
        $validationResult = $this->validateGetRequest($request);
            
        if ($validationResult) {
            return $validationResult;
        }
        
        $data = $this->customerRetargeting($request);
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function sendEmailToTargetedCustomers(Request $request) {

        try {
            foreach ($request->data as $customer) {
                $this->generalController->sendEmail($customer['email'], $request->subject, $request->message);
            }
            return response()->json(['message' => 'Email sent successfully', 'data' => [], 'success' => true], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

    public function sendSMSToTargetedCustomers(Request $request) {
        try {
            foreach ($request->data as $customer) {
                $this->smsController->sendSMS($customer['phone'], $request->message);
            }
            return response()->json(['message' => 'SMS sent successfully', 'data' => [], 'success' => true], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

}
