<?php

namespace App\Http\Controllers\Inventory\Product;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\ItemCategory;
use App\Models\Retails\ItemPrice;
use App\Models\Retails\Items;
use App\Models\Retails\Stock;
use App\Models\Retails\Currency;
use App\Models\Retails\TxnLog;
use App\Models\Retails\Location;
use Validator;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Ramsey\Uuid\Uuid;
use App\Http\Controllers\Settings\Measurement\MeasurementController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use App\Http\Controllers\Inventory\Product\PriceController;
use App\Http\Controllers\Folio\Location\LocationController;
// use DB;
use Illuminate\Support\Facades\DB;



class ProductController extends Controller
{
    protected $measurementController ;
    protected $CurrencyController ;
    protected $ChartOfAccountController ;
    protected $TxnLogController;
    protected $InventoryController;
    protected $priceController;
    protected $locationController;

    public function __construct(MeasurementController $measurementController, CurrencyController $CurrencyController, TxnLogController $TxnLogController, ChartOfAccountController $ChartOfAccountController, InventoryController $InventoryController, PriceController $priceController, LocationController $locationController)
    {
        $this->measurementController = $measurementController;
        $this->CurrencyController = $CurrencyController;
        $this->TxnLogController = $TxnLogController;
        $this->ChartOfAccountController = $ChartOfAccountController;
        $this->InventoryController = $InventoryController;
        $this->priceController = $priceController;
        $this->locationController = $locationController;
    }

   /**
     * Create a new product category or return an error if it already exists.
     *
     * This method checks if a product category with the same name and specific
     * conditions already exists. If it doesn't exist, a new category is created.
     *
     * @param \Illuminate\Http\Request $request The HTTP request containing category data.
     * @return \Illuminate\Http\JsonResponse A JSON response indicating the success or failure of the category creation.
     */

    public function isProductGrouping($client_id) {
        return ItemCategory::where('status', 0)
            ->where('client_id', $client_id)
            ->where('item_class_id', FOUR_STATUS)
            ->first(); 
    }

    public function getItemName($itemId) {
        return Items::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('id', $itemId)
        ->value('name');
    }

    public function isDefaultProductCategory($client_id) {
        return ItemCategory::where('status', 0)
            ->where('client_id', $client_id)
            ->where('name', 'Default Product Category')
            ->first(); 
    }


    public function createProductCategory(Request $request)
    {
        // Define the validation rules for the request data
        $validationRules = [
            'name' => 'required',
            'income_account' => 'required',
        ];

        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);

        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        // Check for existing records that match the given parameters
        $existingCategory = ItemCategory::where([
            'name' => $request->name,
            'status' => 0,
            'user_id' => Auth::user()->id,
            'client_id' => Auth::user()->client_id
        ])->first();
        
        // If a category with the same parameters exists, return an error response
        if (!empty($existingCategory)) {
            return response()->json(['message' => 'The item category already exists', 'success' => false], 400);
        }

        // Define the data for the new item category
        $fields = [
            'id' => Str::uuid()->toString(),
            'item_class_id' => STATUS,
            'name' => $request->name,
            'income_account' => $request->income_account,
            'status' => STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'tm' => Carbon::now()->toTimeString(),
            'date' => Carbon::now()->format('Y-m-d')
        ];

        // Create a new item category
        $itemCategory = ItemCategory::create($fields);

        // Return a success response with the newly created category
        return response()->json(['message' => 'Item Category saved successfully', 'data' => $itemCategory, 'success' => true], 201);
    }





    /**
     * Update an existing product category.
     *
     * This method allows you to edit an existing product category. It first validates
     * the request data, then updates the category if it exists, and returns the result.
     *
     * @param \Illuminate\Http\Request $request The HTTP request containing category data.
     * @param int $id The ID of the category to update.
     * @return \Illuminate\Http\JsonResponse A JSON response indicating the success or failure of the category update.
     */
    public function editProductCategory(Request $request, $id)
    {
        // Define the validation rules for the request data
        $validationRules = [
            'name' => 'required',
            'income_account' => 'required',
        ];

     
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        $itemCategory = ItemCategory::find($id);

        if (!$itemCategory) {
            return response()->json(['message' => 'Item Category not found', 'success' => false], 404);
        }

        $fields = [
            'item_class_id' => STATUS,
            'name' => $request->name,
            'income_account' => $request->income_account,
            'status' => STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'tm' => Carbon::now()->toTimeString(),
            'date' => Carbon::now()->format('Y-m-d')
        ];

        $itemCategory->update($fields);

        return response()->json(['message' => 'Item Category updated successfully', 'success' => true], 200);
    }

    public function getIProductCategory(Request $request)
    {
        $clientId = Auth::user()->client_id;
        $search = $request->input('search');

        $query = ItemCategory::where('client_id', $clientId)
        ->where('status', STATUS)
        ->where('item_class_id', '!=', FOUR_STATUS)
        ->where('name', '!=', 'default product category');
                  

        if (!empty($search)) {
            $query->where('name', 'like', '%' . $search . '%');
        }

        $productCategories = $query->with('chartOfAccount:code,name')->get();

        $transformedCategories = $productCategories->map(function ($category) {
            return [
                'id' => $category->id,
                'app_item_category_id' => $category->app_item_category_id,
                'item_class_id' => $category->item_class_id,
                'name' => $category->name,
                'status' => $category->status,
                'date' => $category->date,
                'tm' => $category->tm,
                'user_id' => $category->user_id,
                'location_id' => $category->location_id,
                'client_id' => $category->client_id,
                'device_id' => $category->device_id,
                'update_date' => $category->update_date,
                'sales_type' => $category->sales_type,
                'income_account' => [
                    'id' => $category->chartOfAccount->code,
                    'name' => $category->chartOfAccount->name,
                ],
                'measurement_category_id' => $category->measurement_category_id,
                'reset' => $category->reset,
                'queued_action_id' => $category->queued_action_id,
                'origin' => $category->origin,
                'created_at' => $category->created_at,
                'updated_at' => $category->updated_at,
                'deleted_at' => $category->deleted_at,
            ];
        });

        return response()->json([
            'message' => 'Product categories fetched successfully',
            'data' => $transformedCategories,
            'success' => true,
        ], 200);
    }

    public function productCatDropdown(Request $request)
    {
       
        $search = $request->input('search'); 

        $query = ItemCategory::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id);
        
        if (!empty($search)) {
            $query->where('name', 'like', '%' . $search . '%');
        }

        $arr = $query->get();

        $response = [];
        foreach ($arr as $array) {
            $response[] = [
                'value' => $array->id,
                'title' => $array->name
            ];
        }

        return response()->json(['message' => 'Product catgeory fetched successfully', 'data' => $response, 'success' => true], 200);
    }


    public function productDropdown(Request $request)
    {
       
        $search = $request->input('search'); 

        $query = Items::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id);
        if (!empty($search)) {
            $query->where('name', 'like', '%' . $search . '%');
        }

        $arr = $query->get();

        $response = [];
        foreach ($arr as $array) {
            $response[] = [
                'value' => $array->id,
                'title' => $array->name,
                'm_cat_id' => $array->m_category_id,
                'item_category_id' => $array->item_category_id
            ];
        }

        return response()->json(['message' => 'Product catgeory fetched successfully', 'data' => $response, 'success' => true], 200);
    }

    public function productList(Request $request)
    {
       
        $search = !empty($request->search) ? " AND p.name LIKE '%$request->search%'" : ""; 
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $limit = "LIMIT $request->count, 50";
        
        $query = "
            SELECT 
                p.name as title,
                p.id as value,
                p.m_category_id as m_cat_id,
                p.item_category_id,

                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', i.m_metric,
                        'title', m.name,
                        'm_category_id', i.m_category_id,
                        'abr', m.abr,
                        'price', i.rate
                    ))

                    FROM " . TBL_ITEM_PRICE . " i

                    LEFT JOIN " . TBL_MEASUREMENT . " m
                    ON m.m_category_id = p.m_category_id AND m.hierarchy=i.m_metric AND m.location_id='$location_id' AND m.client_id='$client_id' AND m.status = 0

                    WHERE i.item_id = p.id AND i.item_id IS NOT NULL AND i.location_id='$location_id' AND i.client_id='$client_id' AND i.status = 0
                    ORDER BY i.m_metric, m.hierarchy ASC
                ) as itemMeasurement

            FROM " . TBL_ITEMS . " p

            WHERE p.client_id = '$client_id' AND p.status = 0 AND p.disabled_status = 0 $search
            ORDER BY p.name ASC
            $limit
        ";

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'itemMeasurement')) {
               
                $row->itemMeasurement = $row->itemMeasurement ? json_decode(stripslashes($row->itemMeasurement), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }


    /**
     * Soft delete a product category by ID.
     *
     * @param int $id The ID of the product category to be deleted.
     * @return \Illuminate\Http\JsonResponse 
     */
    public function deleteProductCategory($id) {
        $record = ItemCategory::find($id);
    
        if (!$record) {
            return response()->json(['message' => 'Record not found', 'success' => false], 404);
        }

          // Check if the category is associated with any items
        $associatedItems = Items::where('item_category_id', $id)->exists();

        if ($associatedItems) {
            return response()->json(['message' => 'Cannot delete category. It is associated with a  product.', 'success' => false], 422);
        }
    
        $record->update(['status' => 1]);
    
        return response()->json(['message' => 'Record deleted successfully', 'success' => true], 200);
    }
    


    public function getProductById($id="")
    {
        $product = Items::find($id);
    
        if (!$product) {
            return response()->json(['message' => 'Product not found', 'success' => false], 404);
        }
    
        $itemPrices = ItemPrice::where('item_id', $id)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->orderBy('m_metric', 'ASC')
        ->get();

        $itemCategory = ItemCategory::find($product->item_category_id);

        $inventoryEvaluation = Location::where('id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->value('inventory_evaluation');
    
        // You can customize the response as needed
        return response()->json(['item_category_name' => $itemCategory ? $itemCategory->name : null, 'product' => $product, 'item_prices' => $itemPrices, 'inventory_evaluation' => $inventoryEvaluation, 'success' => true], 200);
    }
    
    
    
    // public function createProduct(Request $request)
    // {
    //     $validator =   $validationRules = [
    //         'name' => 'required|string',
    //         'item_category_id' => 'required',
    //         'reorder_level' => $request->sales_type == 1 ? 'required' : '',
    //         'currency_id' => !empty($request->qty) ? 'required' : '',
    //         'vat_status' => 'required',
    //         'pricing_information' => 'required|array'
    //     ];

    //     $validator = Validator::make($request->all(), $validationRules);

    //     if ($validator->fails()) {
    //         return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
    //     }

    //     $checkItemCategory = ItemCategory::where('id', $request->item_category_id)->first();

    //     if (!$checkItemCategory) {
    //         return response()->json(['message' => 'Item category not found', 'success' => false], 404);
    //     }

    //     $existingItem = Items::where([
    //         'name' => $request->name,
    //         'status' => 0,
    //         'user_id' => Auth::user()->id,
    //         'client_id' => Auth::user()->client_id
    //     ])->first();

    //     if ($existingItem) {
    //         return response()->json(['message' => 'The item already exists.', 'error' => true], 400);
    //     }

    //     $saveItem = $this->saveItem($request);

    //     if ($saveItem) {
    //         $pricingInfo = $this->savePricingInformation($request, $saveItem);
    //         $uqty = $pricingInfo['uqty'];
    //         $itemPriceFields = $pricingInfo['itemPriceFields'];
    //        $this->saveStockInformation($request, $saveItem, $uqty, $itemPriceFields);
    //     }

    //     return response()->json(['message' => 'Item create successfully', 'success' => true], 201);
    // }

    public function createProduct(Request $request)
    {
        // Start a database transaction
        DB::beginTransaction();
    
        try {
            // Validate the request before starting the transaction
            $validator = Validator::make($request->all(), [
                'name' => 'required|string',
                'item_category_id' => 'required',
                'reorder_level' => $request->sales_type == 1 ? 'required' : '',
                'currency_id' => !empty($request->qty) ? 'required' : '',
                'vat_status' => 'required',
                'pricing_information' => 'required|array'
            ]);
    
            if ($validator->fails()) {
                return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
            }
    
            // Check if the item category exists
            $checkItemCategory = ItemCategory::where('id', $request->item_category_id)->first();
            if (!$checkItemCategory) {
                return response()->json(['message' => 'Item category not found', 'success' => false], 404);
            }
    
            // Check if the item already exists
            $existingItem = Items::where([
                'name' => $request->name,
                'status' => 0,
                'user_id' => Auth::user()->id,
                'client_id' => Auth::user()->client_id
            ])->first();
    
            if ($existingItem) {
                return response()->json(['message' => 'The item already exists.', 'error' => true], 400);
            }
    
            // Save the item
            $saveItem = $this->saveItem($request);
            \Log::channel('slack')->info('Item saved successfully: ' . $saveItem->id);
    
            // Save pricing information
            $pricingInfo = $this->savePricingInformation($request, $saveItem);
            $uqty = $pricingInfo['uqty'];
            $itemPriceFields = $pricingInfo['itemPriceFields'];
    
            // Save stock information
            $this->saveStockInformation($request, $saveItem, $uqty, $itemPriceFields);
    
            // Simulate an error to test rollback (remove this in production)
            // throw new \Exception('Simulated error during item save');
    
            // Commit the transaction if all operations succeed
            DB::commit();
    
            \Log::channel('slack')->info('Transaction completed successfully');
            return response()->json(['message' => 'Item created successfully']);
    
        } catch (\Exception $e) {
            // Log the error and roll back the transaction
            DB::rollBack();
            \Log::channel('slack')->error('Error creating product: ' . $e->getMessage());
            \Log::error('Error creating product: ' . $e->getMessage());
    
            // Return a response with the error message
            return response()->json(['message' => 'Failed to create item: ' . $e->getMessage(), 'error' => true], 500);
        }
    }
    
    protected function saveItem(Request $request)
    {
        try {
            // Generate a UUID for the new item
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
    
            // Prepare the item data
            $itemFields = [
                'id' => $uuidWithoutDashes,
                'item_class_id' => $request->item_class_id ?? 0,
                'item_category_id' => $request->item_category_id,
                'name' => $request->name,
                'reorder_level' => $request->reorder_level,
                'barcode' => $request->barcode,
                'vat_status' => $request->vat_status,
                'sku' => empty($request->sku) ? $this->generateSku($request->name) : $request->sku,
                'description' => $request->description,
                'sales_type' => $request->sales_type,
                'income_account' => $this->getIncomeAccount($request->item_category_id),
                'm_category_id' => $request->m_category_id,
                'status' => STATUS,
                'disabled_status' => STATUS,
                'user_id' => Auth::user()->id,
                'device_id' => $request->device_id,
                'location_id' => Auth::user()->location_id,
                'client_id' => Auth::user()->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d'),
            ];
    
            // Try to create the item in the database
            $item = Items::create($itemFields);
    
            // If item creation fails, throw an exception
            if (!$item) {
                throw new \Exception('Failed to create item');
            }
    
            return $item;
    
        } catch (\Exception $e) {
            // Log the error and rethrow the exception
            \Log::channel('slack')->error('Error saving item: ' . $e->getMessage());
            \Log::error('Error saving item: ' . $e->getMessage());
    
            // Rethrow the exception to trigger rollback
            throw new \Exception('Error saving item: ' . $e->getMessage());
        }
    }
    
  


    

    
    
    
    public function toglleProductVisibility(Request $request) {
        try{
            Items::where('id', $request->id)
                ->where('client_id', Auth::user()->client_id)
                ->update([
                    'disabled_status' => $request->disabled_status == STATUS ? ONE_STATUS : STATUS,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                ]);

            ItemPrice::where('item_id', $request->id)
                ->where('client_id', Auth::user()->client_id)
                ->update([
                    'status' => $request->disabled_status == STATUS ? ONE_STATUS : STATUS,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                ]);

            $message = $request->disabled_status == STATUS ? 'Item disabled successfully' : 'Item enabled successfully';

            return response()->json(['message' => $message, 'success' => true], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error updating item', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }

    public function editProduct(Request $request, $itemId)
    {
        for ($i = 0; $i < count($request->pricing_information); $i++) {
            $priceInfo = $request->pricing_information[$i];
            if($request->cmp > $priceInfo['rate']){
                return response()->json(['message' => 'Cost price cannot be greater than selling price', 'error' => true], 400);
            }
        }
        
        try{
            // Validate the request data
            $validator = Validator::make($request->all(), [
                'name' => 'required|string',
                'item_category_id' => 'required',
                'reorder_level' => 'required|integer',
                'pricing_information' => 'required|array',
            ]);

            if ($validator->fails()) {
                return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
            }

            // Find the item by ID
            $item = Items::find($itemId);

            if (!$item) {
                return response()->json(['message' => 'Item not found', 'success' => false], 404);
            }

            // Check if the new item name conflicts with existing items
            $existingItem = Items::where([
                'name' => $request->name,
                'status' => 0,
                'user_id' => Auth::user()->id,
                'client_id' => Auth::user()->client_id,
                'location_id' => Auth::user()->location_id,
            ])->where('id', '!=', $itemId)->first();

            if ($existingItem) {
                return response()->json(['message' => 'The item already exists.', 'error' => true], 400);
            }

            // Update the item details
            $item->update([
                'item_class_id' => $request->item_class_id ?? 0,
                'item_category_id' => $request->item_category_id,
                'name' => $request->name,
                'reorder_level' => $request->reorder_level,
                'barcode' => $request->barcode,
                'vat_status' => $request->vat_status,
                'sku' => $request->sku,
                'description' => $request->description,
                // 'sales_type' => $request->sales_type,
                'income_account' => $this->getIncomeAccount($request->item_category_id),
                'm_category_id' => $request->m_category_id,
                'tm' => Carbon::now()->toTimeString(),
                'update_date' => Carbon::now()->format('Y-m-d'),
                'disabled_status' => $request->disabled_status
            ]);

            // Update pricing information using the editPricingInformation method
            $this->editPricingInformation($request, $itemId);

            return response()->json(['message' => 'Item updated successfully', 'success' => true], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error updating item', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }




    protected function editPricingInformation($request, $itemId)
    {
          
            // Re-create pricing information based on the request data
            $pricingInformation = $request->pricing_information;
            $count = count($pricingInformation);

            $pm = $this->measurementController->getPrimaryMeasurement($request->m_category_id, Auth::user()->client_id, Auth::user()->location_id);
            $inventoryEvaluation = $this->locationController->getInventoryEvaluation(Auth::user()->location_id, Auth::user()->client_id);

            for ($i = 0; $i < $count; $i++) {
                $priceInfo = $pricingInformation[$i];
                $m_qty =($priceInfo['m_metric'] == 1) ? 1 : $priceInfo['qty'] / $pricingInformation[$i-1]['qty'];

                $cp = ItemPrice::where('item_id', $itemId)
                ->where('primary_measurement', ONE_STATUS)
                ->where('client_id', Auth::user()->client_id)
                ->where('location_id', Auth::user()->location_id)
                ->where('status', STATUS)
                ->value('cp');

                if($this->priceController->countPriceByItemIdAndMetric($itemId, $priceInfo['m_metric']) > 0){
                   
                        ItemPrice::where('item_id', $itemId)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('location_id', Auth::user()->location_id)
                        ->where('m_metric', $priceInfo['m_metric'])
                        ->where('primary_measurement', STATUS)
                        ->update([
                            'cal_qty' => $priceInfo['qty'],
                            'qty' => $m_qty,
                            'rate' => $priceInfo['rate'],
                            'cp' => 0,
                            'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                        ]);

                        ItemPrice::where('item_id', $itemId)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('location_id', Auth::user()->location_id)
                        ->where('m_metric', $priceInfo['m_metric'])
                        ->where('primary_measurement', ONE_STATUS)
                        ->update([
                            'cal_qty' => $priceInfo['qty'],
                            'qty' => $m_qty,
                            'rate' => $priceInfo['rate'],
                            'cp' => ($inventoryEvaluation == ONE_STATUS && $request->cmp) ? $request->cmp : $cp,
                            'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                        ]);
                        
                } else {
                    $uuid = Str::uuid()->toString();
                    $uuidWithoutDashes = str_replace('-', '', $uuid);
    
                    $itemPriceFields = [
                        'id' => $uuidWithoutDashes,
                        'rate' => $priceInfo['rate'],
                        'item_id' => $itemId,
                        'm_metric' => $priceInfo['m_metric'],
                        'primary_measurement' => ($priceInfo['m_metric'] == $pm) ? ONE_STATUS : STATUS,
                        'qty' => $m_qty,
                        'cal_qty' => $priceInfo['qty'],
                        'control_user_id' => STATUS,
                        'control_id' => STATUS,
                        'item_category_id' => $request->item_category_id,
                        'item_class_id' => $request->item_class_id ?? 0,
                        'm_category_id' => $request->m_category_id,
                        'default_price' => 1,
                        'status' => STATUS,
                        'cp' => ($inventoryEvaluation == ONE_STATUS && $priceInfo['m_metric'] == $pm && $request->cmp) ? $request->cmp : 0,
                        'user_id' => Auth::user()->id,
                        'location_id' => Auth::user()->location_id,
                        'client_id' => Auth::user()->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d'),
                        'update_date' => Carbon::now()->format('Y-m-d'),
                    ];
    
                    // Assign the created item price to $saveItemPrice
                    ItemPrice::create($itemPriceFields);
                }
            }
            
          
    }



    public function getIncomeAccount($itemCategoryId)
    {
        $itemCategory = ItemCategory::where('id', $itemCategoryId)->first();

        if ($itemCategory) {
            return $itemCategory->income_account;
        } else {
            return null; 
        }
    }

    
    

    protected function generateSku($name)
    {
        $randomNumber = mt_rand(100, 999);
        $prefix = substr($name, 0, 2);

        return $prefix . $randomNumber . now()->timestamp;
    }

    protected function savePricingInformation(Request $request, $saveItem)
    {
        $pricingInformation = $request->pricing_information;
        $count = count($pricingInformation);
    
        $previousQty = 0;
        $pm = $this->measurementController->getPrimaryMeasurement($request->m_category_id, Auth::user()->client_id, Auth::user()->location_id);
        $uqty = 0; // Initialize $uqty
        $metricArray = [];
        // Initialize $saveItemPrice outside the loop
        $saveItemPrice = array();
        $inventoryEvaluation = $this->locationController->getInventoryEvaluation(Auth::user()->location_id, Auth::user()->client_id); 

        for ($i = 0; $i < $count; $i++) {

            $priceInfo = $pricingInformation[$i];

            // accumulating the metric into an array to be used later outside this loop below
            array_push($metricArray, $priceInfo['m_metric']);

            $m_qty = ($priceInfo['m_metric'] == 1) ? 1 : $priceInfo['qty'] / $pricingInformation[$i-1]['qty'];
    
            if ($priceInfo['m_metric'] == $request->m_metric) {
                $uqty = $priceInfo['qty'] * $request->qty;
            }

            $uuid = Str::uuid()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
    
            $itemPriceFields = [
                'id' => $uuidWithoutDashes,
                'rate' => $priceInfo['rate'],
                'item_id' => $saveItem->id,
                'm_metric' => $priceInfo['m_metric'],
                'primary_measurement' => ($priceInfo['m_metric'] == $pm) ? ONE_STATUS : STATUS,
                'qty' => $m_qty,
                'cal_qty' => $priceInfo['qty'],
                'device_id' => $request->device_id,
                'control_user_id' => $request->control_user_id,
                'control_id' => $request->control_id,
                'item_category_id' => $request->item_category_id,
                'item_class_id' => $request->item_class_id ?? 0,
                'm_category_id' => $request->m_category_id,
                'default_price' => 1,
                'cp' => ($inventoryEvaluation == ONE_STATUS && $priceInfo['m_metric'] == ONE_STATUS) ? $request->cmp : null,
                'status' => STATUS,
                'user_id' => Auth::user()->id,
                'location_id' => Auth::user()->location_id,
                'client_id' => Auth::user()->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d'),
            ];
    
            // Assign the created item price to $saveItemPrice
            $saveItemPrice = ItemPrice::create($itemPriceFields);
    
            
        }

        // saving other measurement and price that were left out by the user during item creation
        $prices = $this->measurementController->getMeausrementByCategory($request->m_category_id);

        foreach($prices as $price) {
            $metric = $price['hierarchy'];
            if (!in_array($metric, $metricArray)) {
                $uuid = Str::uuid()->toString();
                $uuidWithoutDashes = str_replace('-', '', $uuid);

                ItemPrice::create([
                    'id' => $uuidWithoutDashes,
                    'rate' => STATUS,
                    'item_id' => $saveItem->id,
                    'm_metric' => $metric,
                    'primary_measurement' => STATUS,
                    'qty' => STATUS,
                    'cal_qty' => STATUS,
                    'device_id' => $request->device_id,
                    'control_user_id' => $request->control_user_id,
                    'control_id' => $request->control_id,
                    'item_category_id' => $request->item_category_id,
                    'item_class_id' => $request->item_class_id ?? 0,
                    'm_category_id' => $request->m_category_id,
                    'default_price' => 1,
                    'status' => STATUS,
                    'user_id' => Auth::user()->id,
                    'location_id' => Auth::user()->location_id,
                    'client_id' => Auth::user()->client_id,
                    'tm' => Carbon::now()->toTimeString(),
                    'date' => Carbon::now()->format('Y-m-d'),
                    'update_date' => Carbon::now()->format('Y-m-d'),
                ]);
            }
        }    

    return ['uqty' => $uqty, 'itemPriceFields' => $saveItemPrice];
}
    
    


    protected function saveStockInformation(Request $request, $saveItem, $uqty, $itemPriceFields)
    {
       

        if ($request->cp > 0 && $request->qty > 0) {
            $amt = $request->cp * $request->qty;
            // $currencyUnit = Currency::where('id', $request->currency_id)->first()->unit;
            $currencyUnit = $this->CurrencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $request->currency_id);
            $token = Uuid::uuid4()->toString();
            $txn_date = Carbon::now()->format('Y-m-d');

            $this->InventoryController->insertStock($token, $txn_date, $saveItem->id, "", $request->item_category_id, $request->qty, "", $request->currency_id, $request->m_metric, $request->m_category_id, $request->store_id, EFFECT_DEBIT, null, $amt);

            ItemPrice::where('item_id', $saveItem->id)
                ->where('client_id', Auth::user()->client_id)
                ->where('location_id', Auth::user()->location_id)
                ->where('primary_measurement', ONE_STATUS)
                ->update([
                    'cp' => $request->cp,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                ]);

            $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_STOCK, CHART_SUSPENSE, Auth::user()->client_id);

            $this->TxnLogController->createTxnLog($accLegs, $amt, $token, $currencyUnit, $txn_date, "", 'Being Previous Balance', "", 'pb', $request->currency_id);
        }
    }






    public function deleteProduct($id)
    {
        $itemRecord = Items::find($id);
        $checkStock = Stock::where('item_id', $id)
        ->where('control_id', '!=', ONE_STATUS)
        ->first();
    
        if (!$itemRecord) {
            return response()->json(['message' => 'Record not found', 'success' => false], 404);
        }
    
        if ($checkStock) {
            return response()->json(['message' => 'Item can not be deleted', 'success' => false], 404);
        }
    
        $itemRecord->update(['status' => 1]);
    
        ItemPrice::where('item_id', $id)->update(['status' => 1]);
    
        $stockRecord = Stock::find($id);
        if ($stockRecord) {
            $stockRecord->update(['status' => 1, 'control_id' => 1]);
        }
    
        return response()->json(['message' => 'Records deleted successfully', 'success' => true], 200);
    }
    

    public function getProductData(Request $request) {
        $client_id=Auth::user()->client_id;
        $search = !empty($request->search) ? " AND name LIKE '%$request->search%'" : "";
        $limit = "LIMIT $request->count, 50";

        $query = "
            SELECT
                name as title,
                id as value
            FROM " . TBL_ITEMS . "
            WHERE client_id='$client_id' AND status = " . STATUS . " $search
            $limit
        ";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }
    
    public function getDefaultItemCategory($client_id) {
        return ItemCategory::where('name', 'default product category')
        ->where('client_id', $client_id)
        ->where('status', STATUS)
        ->get();
    }

    public function getItemCategoryByItemId($item_id) {
        return Items::where('id', $item_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->value('item_category_id');
    }

    public function getItemClassByItemId($item_id) {
        return Items::where('id', $item_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->value('item_class_id');
    }

    public function getItemMeasurementCategoryByItemId($item_id) {
        return Items::where('id', $item_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->value('m_category_id');
    }

    public function countItemIdByName($name) {
        return Items::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('name', $name)
        ->count();
    }

    public function countItemCategoryByName($name) {
        return ItemCategory::where('name', $name)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->count();
    }
}
