<?php

namespace App\Http\Requests;

use App\Enums\LogType;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Contracts\Validation\Validator;
use App\Rules\LoginPermission;
use App\Enums\OriginEnum; // Custom Enum for origin validation
use App\Services\Contracts\LogServiceInterface;
use Illuminate\Http\Exceptions\HttpResponseException;
use Illuminate\Validation\Rule;

class LoginRequest extends FormRequest
{
    protected $logService;

    public function __construct()
    {
        parent::__construct();

        // Resolve LogService manually
        $this->logService = app(LogServiceInterface::class);
    }

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {

        return true;
    }

    /**
     * Sanitize input before validation.
     *
     * @return void
     */
    protected function prepareForValidation()
    {
        $this->merge([
            'username' => strtolower(trim($this->username)), // Trim and convert username/email to lowercase
            'password' => strtolower(trim($this->password)),
            'macAddress' => isset($this->macAddress) ? trim($this->macAddress) : null, // Trim macAddress if present
            'origin' => is_numeric($this->origin) ? (int) $this->origin : $this->origin
        ]);
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'password' => 'required|min:4', // Minimum of 4 characters
            'origin' => [
                'required',
                Rule::in(OriginEnum::values()), // Validate against the custom enum
            ],
            'macAddress' => [
                'nullable', // Only required for specific origins
                'required_if:origin,2,1', // Required when origin is desktop or web
                'string',
                'max:255',
            ],
            'username' => [
                'required',
                'string',
                new LoginPermission($this->input('origin') ?? 0), // Check user existence based on username/email
            ],
        ];
    }

    /**
     * Custom error messages for validation rules.
     *
     * @return array
     */
    public function messages()
    {
        return [
            'username.required' => 'A username or email address is required.',
            'username.string' => 'The username must be a valid text.',
            'password.required' => 'A password is required.',
            'password.min' => 'Password must be at least 4 characters.',
            'origin.required' => 'The origin is required.',
            'origin.in' => 'The origin must be one of the following: web (3), mobile (1), desktop (2).',
            'macAddress.required_if' => 'The macAddress is required for mobile or desktop origins.',
            'macAddress.string' => 'The macAddress must be a valid string.',
            'macAddress.max' => 'The macAddress may not be greater than 255 characters.',
        ];
    }
    /**
     * Handle failed validation.
     *
     * @param  \Illuminate\Contracts\Validation\Validator  $validator
     * @throws \Illuminate\Validation\ValidationException
     */

    protected function failedValidation(Validator $validator)
    {
        // Log validation failure using LogService
        $this->logService->support('Validation Failed', LogType::WARNING->value, [
            'message' => $validator->errors()->first(),
            'input' => $this->input(),
            'route' => $this->path(),
            'ip_address' => $this->ip(),

        ]);

        // Extract the first error message from the validation errors
        $message = $validator->errors()->first();
        $errors = $validator->errors()->toArray(); // Convert errors to an array
        $code = $errors['username'][1] ?? 409;
        $action = $errors['username'][2] ?? null;

        // Prepare the response in the required format
        $response = [
            'success' => false,
            'message' => $message, // Extract the message or fallback to default
            'action' => $action // Optional action, default to null
        ];

        // Throw the response as an exception with a 422 status code
        throw new HttpResponseException(
            response()->json($response, $code) // Use provided code or default to 422
        );
    }
}
