<?php

namespace App\Logging;

use Monolog\Handler\AbstractProcessingHandler;
use Monolog\Logger;
use GuzzleHttp\Client;

class TeamsWebhookHandler extends AbstractProcessingHandler
{
    protected $webhookUrls;

    public function __construct(array $webhookUrls, $level = Logger::DEBUG, bool $bubble = true)
    {
        $this->webhookUrls = $webhookUrls;
        parent::__construct($level, $bubble);
    }

    protected function write(array $record): void
    {
        // Extract the channel from the context, default to 'default'
        $channel = $record['context']['channel'] ?? 'default';
        $webhookUrl = $this->webhookUrls[$channel] ?? $this->webhookUrls['default'];

        $client = new Client();

        // Create the Teams card-style message
        $message = $this->createCardMessage($record);

        // Send the log to the appropriate Teams channel
        $client->post($webhookUrl, [
            'json' => $message,
        ]);
    }

    protected function createCardMessage(array $record): array
    {
        // Map error levels to color codes
        $errorLevelColor = match (strtolower($record['level_name'])) {
            'critical' => 'Attention',  // Red for critical errors
            'warning' => 'Warning',    // Orange for warnings
            'info' => 'Good',          // Blue for informational logs
            default => 'Default',      // Grey for other levels
        };

        // Core card structure
        $card = [
            'type' => 'message',
            'attachments' => [
                [
                    'contentType' => 'application/vnd.microsoft.card.adaptive',
                    'content' => [
                        'type' => 'AdaptiveCard',
                        'version' => '1.4',
                        'body' => [
                            // Error level tag
                            [
                                'type' => 'TextBlock',
                                'text' => strtoupper($record['level_name']),
                                'weight' => 'Bolder',
                                'size' => 'Small',
                                'horizontalAlignment' => 'Right',
                                'color' => $errorLevelColor,
                            ],
                            // Main message text
                            [
                                'type' => 'TextBlock',
                                'text' => $record['message'],
                                'weight' => 'Bolder',
                                'size' => 'Medium',
                                'wrap' => true,
                            ],
                        ],
                    ],
                ],
            ],
        ];

        // Add dynamic context details
        if (!empty($record['context'])) {
            $facts = [];

            foreach ($record['context'] as $key => $value) {
                if (!is_array($value)) {
                    $facts[] = ['title' => ucfirst(str_replace('_', ' ', $key)) . ':', 'value' => (string) $value];
                }
            }

            if (!empty($facts)) {
                $card['attachments'][0]['content']['body'][] = [
                    'type' => 'FactSet',
                    'facts' => $facts,
                ];
            }
        }

        // Add stack trace if available
        if (!empty($record['context']['trace'])) {
            $card['attachments'][0]['content']['body'][] = [
                'type' => 'TextBlock',
                'text' => 'Stack Trace:',
                'weight' => 'Bolder',
                'size' => 'Small',
            ];
            $card['attachments'][0]['content']['body'][] = [
                'type' => 'TextBlock',
                'text' => $record['context']['trace'],
                'wrap' => true,
                'spacing' => 'Small',
            ];
        }

        return $card;
    }
}
