<?php

namespace App\Providers;

use Illuminate\Support\Facades\Schema;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Validator;
use App\Models\Retails\ItemCategory;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Events\QueryExecuted;
use App\Models\Retails\QueueLog;
use App\QueryBuilders\Contracts\QueryBuilderInterface;
use App\QueryBuilders\UserRightsBuilder;
use Illuminate\Support\Str;
use Carbon\Carbon;
use App\Services\Contracts\PasswordServiceInterface;
use App\Services\PasswordService;
use App\Repositories\Contracts\CustomQueryRepositoryInterface;
use App\Repositories\CustomQueryRepository;
use App\Services\Contracts\LogServiceInterface;
use App\Services\LogService;
use Illuminate\Support\Facades\Log;

class AppServiceProvider extends ServiceProvider
{

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        $this->app->bind(PasswordServiceInterface::class, PasswordService::class);
        $this->app->bind(CustomQueryRepositoryInterface::class, CustomQueryRepository::class);
        $this->app->bind(QueryBuilderInterface::class, UserRightsBuilder::class);
        $this->app->singleton(LogServiceInterface::class, function ($app) {
            return new LogService();
        });
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */


    public function boot()
    {
        register_shutdown_function(function () {
            $error = error_get_last();

            if ($error && in_array($error['type'], [E_ERROR, E_CORE_ERROR, E_COMPILE_ERROR, E_PARSE])) {
                Log::channel('teams')->critical('Fatal PHP Error', [
                    'type' => $error['type'],
                    'message' => $error['message'],
                    'file' => $error['file'],
                    'line' => $error['line'],
                    'channel' => 'critical',
                ]);
            }
        });

        // Set the default string length for schema
        Schema::defaultStringLength(191);

        DB::enableQueryLog();

        // Listen for database queries
        DB::listen(function ($query) {

            if ($query->time > 1000) { // Log slow queries over 1 second
                Log::channel('teams')->critical('Slow Query Detected', [
                    'sql' => $query->sql,
                    'bindings' => $query->bindings,
                    'time' => $query->time,
                    'channel' => 'critical',
                ]);
            }

            $sql = strtolower(trim($query->sql));

            // Check if this is an INSERT, UPDATE, or DELETE query
            if ((Str::startsWith($sql, 'insert into') || Str::startsWith($sql, 'update') || Str::startsWith($sql, 'delete from'))
                && !Str::contains($sql, 'queue_log')
            ) {
                // Attempt to extract the table name from the query
                $tableName = $this->extractTableName($sql);

                // Check if the table is in the list of tables to log
                if (in_array($tableName, $this->tableArray())) {
                    $identifier = $this->extractIdentifier($sql, $query->bindings);

                    // Construct the full query with bindings
                    $fullSql = $this->replaceBindings($sql, $query->bindings);

                    // Log the query with the extracted table name
                    $user = Auth::user();
                    $user_id = $user->id ?? session('user_id', null);

                    if ($user_id) {
                        QueueLog::create([
                            'table_name' => $tableName,
                            'data_result' => $fullSql,
                            'identifier' => $identifier,
                            'token' => '',
                            'sync_status' => ONE_STATUS,
                            'source' => ONE_STATUS,
                            'device_id' => STATUS,
                            'user_id' => Auth::user()->id ?? session('user_id', 'default_value'),
                            'location_id' => Auth::user()->location_id ?? session('location_id', 'default_value'),
                            'client_id' => Auth::user()->client_id ?? session('client_id', 'default_value'),
                            'date' => Carbon::now()->format('Y-m-d'),
                            'tm' => Carbon::now()->toTimeString(),
                            'last_pull' => Carbon::now()->format('Y-m-d H:i:s'),
                            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                            'status' => STATUS
                        ]);
                    }
                }
            }
        });
    }


    private function replaceBindings($query, $bindings)
    {
        // Prepare the values from bindings
        $preparedBindings = array_map(function ($binding) {
            return is_numeric($binding) ? $binding : "'" . addslashes($binding) . "'";
            // return is_numeric($binding) ? $binding : "\"" . addslashes($binding) . "\"";
        }, $bindings);

        // Replace all ? placeholders with the actual values from bindings
        $query = preg_replace_callback('/\?/', function () use (&$preparedBindings) {
            return array_shift($preparedBindings);
        }, $query);

        // Define the SQL keywords to transform to uppercase in one regex pattern
        $keywordsPattern = '\b(' . implode('|', ['insert into', 'values', 'update', 'set', 'delete from', 'where']) . ')\b';

        // Replace each keyword with its uppercase version
        $query = preg_replace_callback('/' . $keywordsPattern . '/i', function ($matches) {
            return strtoupper($matches[0]);
        }, $query);

        // Remove backticks
        $query = str_replace("`", "", $query);

        return $query;
    }







    private function extractIdentifier($sql, $bindings)
    {
        if (Str::startsWith($sql, 'insert into')) {
            // For INSERT, the ID might be the first value in the values list.
            // This assumes the ID is the first field in your INSERT statements.
            preg_match("/values\s*\((.*?)\)/i", $sql, $matches);
            if (isset($matches[1])) {
                $values = explode(',', $matches[1]);
                $firstValuePlaceholder = trim($values[0]); // This should be '?' for most cases
                if ($firstValuePlaceholder === '?') {
                    // Replace the placeholder with the actual value from the bindings
                    return $bindings[0] ??  Str::uuid()->toString();
                }
            }
        } elseif (Str::startsWith($sql, 'update') || Str::startsWith($sql, 'delete from')) {
            // Similar logic for UPDATE and DELETE where the ID is in the WHERE clause
            if (preg_match('/where `id` = \?/i', $sql)) {
                $wherePosition = strpos($sql, 'where');
                $placeholdersBeforeWhere = substr_count(substr($sql, 0, $wherePosition), '?');
                // return $bindings[$placeholdersBeforeWhere] ?? 'default-identifier';
                return  Str::uuid()->toString(); // 
            }
        }

        return  Str::uuid()->toString(); // Generate new identifier if no identifier is found
    }


    private function extractTableName($sql)
    {
        if (Str::startsWith($sql, 'insert into')) {
            preg_match('/insert into `?([^ `]+)`?/i', $sql, $matches);
        } elseif (Str::startsWith($sql, 'update')) {
            preg_match('/update `?([^ `]+)`?/i', $sql, $matches);
        } elseif (Str::startsWith($sql, 'delete from')) {
            preg_match('/delete from `?([^ `]+)`?/i', $sql, $matches);
        }

        return $matches[1] ?? 'unknown'; // Return the table name or 'unknown' if not found
    }

    public function tableArray()
    {
        return [
            'bank',
            'bank_account',
            'basic_log',
            'chart_cate',
            'chart_of_accounts',
            'credit_log',
            'currency',
            'currency_cate',
            'customers',
            'estimate',
            'items',
            'item_category',
            'item_class',
            'item_grouping',
            'item_log',
            'item_price',
            'loyalty_log',
            'loyalty_management',
            'reminder',
            'sales_rep',
            'sms_setting',
            'stock',
            'stock_adjustment',
            'stock_take',
            'store',
            'tmp_item_log',
            'tmp_log',
            'tmp_txn_log',
            'txn_log',
            'vat',
            'outlet',
            'tables',
            'table_category',
            'buildings',
            'floors',
            'rooms',
            'lodge_log',
            'recipe',
            'recipe_ingredients',
            'reservation',
            'club',
            'club_benefit_table',
            'club_member',
            'bar',
            'checkout_log',
            'corporate_rate',
            'event_ticket',
            'event_ticket_log',
            'events',
            'kitchen_inventory',
            'outlet_requisition',
            'restaurant',
            'room_change',
            'room_flag',
            'room_type',
            'shift_log',
            'shift_managements',
            'space',
            'table_reservation',
            'ticket',
            'cashlog',
            'charged',
            'location',
            'receipt_settings'
        ];
    }
}
