<?php

namespace App\Rules;

use App\Enums\OriginEnum;
use App\Enums\PermissionStatus;
use App\Enums\UserStatus;
use App\Enums\VerificationStatus;
use Illuminate\Contracts\Validation\Rule;
use App\Models\Account\User;
use App\Repositories\Contracts\UserRepositoryInterface;
use Illuminate\Http\Request;
use App\Services\UserCacheService;

class LoginPermission implements Rule
{

    protected int $origin;
    protected array $error = [];
    protected UserRepositoryInterface $userRepository;


    /**
     * Create a new rule instance.
     *
     * @return void
     */
    public function __construct(int $origin)
    {
        $this->origin = $origin;

        // Resolve the dependency manually
        $this->userRepository = app(UserRepositoryInterface::class);
    }

    /**
     * Determine if the validation rule passes.
     *
     * @param  string  $attribute
     * @param  mixed  $value
     * @return bool
     */
    public function passes($attribute, $value)
    {
        $user = $this->userRepository->findByEmailOrUsername($value);
        // \Log::info('User login attempt',['user' => $user]);

        if (!$user) {
            $this->error = [
                'message' => 'User does not exist.',
                'code' => 404, // Not Found
                'action' => null, // No specific action needed
            ];
            return false;
        }

        if ($user->status === UserStatus::Inactive->value) {
            $this->error = [
                'message' => 'Your account has been disabled.',
                'code' => 403, // Forbidden
                'action' => 'Contact support to reactivate your account.',
            ];
            return false;
        }

        // Check for verification status
        if ($this->origin !== OriginEnum::WEB->value && $user->verification_status === VerificationStatus::NotVerified->value) {
            $this->error = [
                'message' => 'Your account(email/phonenumber) needs to be verified.',
                'code' => 403,
                'action' => 'Verify',
            ];

            return false;
        }

        // Cache the user data for later usage
        UserCacheService::getInstance()->setUser($user);


        // return match ($this->origin) {
        //     1 => $user->allowed_on_mobile == 1,
        //     2 => $user->allowed_on_desktop == 1,
        //     3 => $user->allowed_on_web == 1,
        //     default => false,
        // };
        return $this->isAllowedForOrigin(OriginEnum::fromValue($this->origin), $user);

        return true;
    }

    public function isAllowedForOrigin(OriginEnum $origin, User $user): bool
    {
        return match ($origin) {
            OriginEnum::MOBILE => $user->allowed_on_mobile == 1,
            OriginEnum::DESKTOP => $user->allowed_on_desktop == 1,
            OriginEnum::WEB => $user->allowed_on_web == 1,
            default => false,
        };
    }

    /**
     * Get the validation error message.
     *
     * @return array
     */
    public function message()
    {
        return $this->error;
    }
}
