<?php

namespace App\Services;

use App\Services\Contracts\PasswordServiceInterface;

class PasswordService implements PasswordServiceInterface
{
    /**
     * Hash a password using a salt.
     *
     * @param string $password The password to hash.
     * @param string $salt The salt to use in hashing.
     * @return string The hashed password.
     */
    public function hashPass(string $password, string $salt): string
    {
        return hash("SHA512", base64_encode(str_rot13(hash("SHA512", str_rot13($salt . $password)))));
    }

    /**
     * Generate a random salt.
     *
     * @return string The generated salt.
     */
    public function generateSalt(): string
    {
        $length = 45;
        return base64_encode(random_bytes(ceil(0.75 * $length)));
    }

    /**
     * Verify if a password matches a hashed password when combined with a salt.
     *
     * @param string $inputPassword The password input to verify.
     * @param string $storedHash The stored hashed password to compare against.
     * @param string $salt The salt used in hashing the stored password.
     * @return bool True if the password is valid, false otherwise.
     */
    public function verifyPassword(string $inputPassword, string $storedHash, string $salt): bool
    {
        // \Log::info('User data:', ['inputPassword' => $inputPassword, 'salt' => $salt]);
        $hashedInput = $this->hashPass($inputPassword, $salt);

        $storedHash = (string) trim($storedHash);
        $hashedInput = (string) trim($hashedInput);

        return hash_equals(trim($storedHash), trim($hashedInput));
    }
}
